//
//  AASDirectorySubscription.h
//
//  Created by andreas on Fri Nov 16 2001.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


//  Most interesting in this header file is probably the subscriber notification protocol at the bottom.  Don't miss it.
//

#import <Foundation/Foundation.h>
#import "AASQueuedObjectOperation.h"
#import "AASFilesystemSubscription.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/event.h>
#include <sys/time.h>
#include <errno.h>
#include <string.h>
#include <inttypes.h>
#include <fts.h>








#pragma mark -

@class AASFileItem;
@class AASFileLoadOperation;

@interface AASDirectorySubscription : AASFilesystemSubscription <AASQueuedObjectOperationDelegateProtocol> {	
	NSURL *lastKnownURL;
	AASFileItem *directoryFile;
    NSMapTable *directoryInfo;
	NSDictionary *childrenPaths;
	
	AASFileLoadOperation *currentLoadOperation;
    BOOL stopLoading;
    
    BOOL asynchronous;
    
    BOOL shouldCheck;
	BOOL paused;
	
    NSMapTable *_container;
	NSTimer *removalTimer;
	NSTimer *removalCheckTimer;
}



#pragma mark -
#pragma mark Creating Subscriptions



/* Gets the AASDirectorySubscription object for a particular directory and optionally adds a subscriber.  This is meant to be done by an AASFileManager instance, but it can be used independently if you want to for some reason.  When asked to check the directory, it checks the kernel queue to see if anything has changed since the last check, and if so, sends the subscriber(s) an array with information on what files changed.  This tracks the directory's immediate contents only, it doesn't recursively check subdirectories. Returns nil if init fails for any reason.  If asynchronous is YES, the contents of the directory are loaded and the file objects created in a separate thread, and the contents are checked for changes in a separate thread; the initial return from methods like -files may be empty arrays or only some of the actual directory contents. Subscribers will periodically recieve AASFileAppeared notifications from the main thread to update while asynchronous loading is occurring. Subscribers can check the -isLoading method during notifications if desired; a final notification will be sent when loading has completed, so checking -isLoading during the notifications will have an opportunity to return NO. */
+ (AASDirectorySubscription *)directorySubscriptionForDirectory:(AASFileItem *)dir addSubscriber:(id<AASFilesystemSubscriptionProtocol>)sub;


/* Inits an AASDirectorySubscription instance. Used by directorySubscriptionForDirectory:addSubscriber: to create new objects when necessary. Do not use it directly. */
- (id)initForDirectory:(AASFileItem *)dir;



#pragma mark -
#pragma mark Subscription Information

/* Returns the path of the directory whose contents this AASDirectorySubscription is watching. */
@property(readonly) NSString * path;

/* Returns the url of the directory this AASDirectorySubscription is watching. */
@property(readonly, retain) NSURL * url;

/* Tells whether the reciever is still valid or not (it isn't valid anymore if the directory it is checking was deleted). */
@property(readonly) BOOL isValid;

/* Tells whether or not the AASDirectorySubscription is currently loading files from disk (only happens if asynchronous was set to YES). */
@property(readonly) BOOL isLoading;

/* Returns an array of file objects for all the visible files in the directory. */
@property(readonly) NSArray * files;

/* Returns the file object for the directory whose contents this AASDirectorySubscription is watching. */
@property(readonly) id directoryFile;



#pragma mark -
#pragma mark Subscription Actions


/* If the AASDirectorySubscription is set to load asynchronously and hasn't begun or hasn't completed loading the directory's contents, this stops the asynchronous process and finishes the loading immediately. Everyone's in a hurry these days. */
- (void)loadImmediately;







@end



