//
//  GBHotkeyButton.h
//  GBBasicsKit
//
//  Created by Andreas Schwarz on 11/30/09.
//  Copyright 2009 immaterial. All rights reserved.
//

#import <Cocoa/Cocoa.h>




extern NSString *GBHotkeyModifierKeysKey;
extern NSString *GBHotkeyKeyCodeKey;
extern NSString *GBHotkeyKeyMenuStringKey;


#define GB_UNKNOWN_KEYCODE (unsigned short)-1



// Function for getting string value of key code. Returns nil if unknown or error.
NSString *GBGetStringForKeyCode(unsigned short keyCode);

// Gets the display string for the key code
NSString *GBGetDisplayStringForKeyCode(unsigned short keyCode, BOOL specifyNumberpad);

// Gets the keycode for a particular character, if possible. There are multiple keycodes that can resolve to the same string and there is no gaurantee which keycode will be returned.
unsigned short GBGetKeyCodeForString(NSString *character);


@interface GBHotkeyButton : NSButton {
	NSDictionary *hotkey;
	NSDictionary *displayHotkey;
	BOOL active;
	BOOL clearHotkey;
	BOOL requireModifiers;
	BOOL requireMenuModifiers;
	BOOL menuStyle;
	NSTimer *messageTimer;
}

// Set/get hotkey
@property(copy) NSDictionary * hotkey;

// Require at least one modifier key (command, option, shift, or control)
@property(assign) BOOL requireModifiers;

// Require at least one modifier key required by NSMenuItems (command, or control)
@property(assign) BOOL requireMenuModifiers;

// menuStyle = YES displays the character string rather than the character determined by the keyCode
@property(assign) BOOL menuStyle;


@end


@interface NSDictionary (GBHotkeyUtilities)

// Characters can be nil, keyCode can be GB_UNKNOWN_KEYCODE, but not both at once
+ (NSDictionary *)hotkeyWithModifiers:(NSUInteger)modifiers keyCode:(unsigned short)keyCode character:(NSString *)character;
+ (NSDictionary *)hotkeyWithCommand:(BOOL)command shift:(BOOL)shift option:(BOOL)option control:(BOOL)control  keyCode:(unsigned short)keyCode character:(NSString *)character;

- (NSUInteger)hotkeyModifiers;
- (unsigned short)hotkeyKeyCode;
- (NSString *)hotkeyCharacterUsingMenuStyle:(BOOL)menuStyle;
- (NSString *)hotkeyDisplayModifiers;
- (NSString *)hotkeyDisplayCharacterUsingMenuStyle:(BOOL)menuStyle;
- (NSString *)hotkeyDisplayStringUsingMenuStyle:(BOOL)menuStyle;

- (BOOL)hotkeyMatchesEvent:(NSEvent *)theEvent;

@end

@interface NSMenuItem (GBHotkeyUtilities)

// Sets key equivalent and modifier mask to appropriate values based on hotkey
- (void)setHotkey:(NSDictionary *)hotkey;

// Like setHotkey: but sets key equivalent based on the keyCode rather than the stored string. Use this in situations where you're using a keyCode based hotkey (where you want to use a specific key on the keyboard rather than a specific character) but still want a menu item for it.
- (void)setHotkeyByKeycode:(NSDictionary *)hotkey;

@end








