/*
 *  GBUtilityFunctions.h
 *  
 *
 *  Created by Andreas Schwarz on Mon Jun 03 2002.
 *  Copyright (c) 2002 Andreas Schwarz. All rights reserved.
 *
 */

#ifndef __GBUTILITYFUNCTIONS__
#define __GBUTILITYFUNCTIONS__

#include <sys/types.h>
#include <sys/stat.h>

#import <Cocoa/Cocoa.h>

@class GBFileItem;



/* Miscellaneous functions */

	/* Returns an array of UTIs for files which we'll edit metadata (EXIF, IPTC, etc.) for. */
NSArray * GBEditableMetadataImageTypes();

	/* Takes a size (ie. image size) and returns a size scaled proportionally to 'max' on the longest side */
NSSize GBProportionalScaleToMaxSize(NSSize inSize, NSInteger max);


/* Takes an array of UTIs (such as you'd get from +[NSImage imageTypes]) and returns YES if inUTI conforms to one of those UTIs. */
BOOL GBUTTypeConformsToArray(CFStringRef inUTI, NSArray *testUTIs);
    


    /* Replaces all occurances of a specified substring (badString) in searchString with a new string (newString).  Used by the "GBCreateCleanFileName" function to replace illegal characters with dashes */
void GBReplaceAllOccurrancesOfString(CFMutableStringRef searchString, CFStringRef badString, CFStringRef newString);
    
    /* Takes an NSString that should represent a file name (not an entire path, because it replaces slashes) and replaces any illegal characters with dashes (good to use when the user renames a file).  Currently, it will replace: "/", ":", and "._" */
NSString * GBCreateCleanFileName(NSString *filename);

    /* Uses appleevents to tell the Finder to open a new window */
void GBOpenNewFinderWindow();

    /* Uses FSFindFolder to find the appropriate folder and return the appropriate path */
NSString *GBUserPicturesDirectoryPath();

void GBSetFileExtensionHidden(NSString *file, BOOL hidden);

/* Drawing functions */

    /* Draws a message box, kind of like a message window, except inside a view. */
void GBDrawMessageBox(NSString *message, NSRect frame, NSView *view);

    /* Draws a message box with a file's name and icon inside a view. */
void GBDrawMessageBoxWithFile(NSString *message, NSRect frame, NSView *view, GBFileItem *file);


/* Removes thumbnail data from resource fork of a file */
BOOL GBRemoveThumbnailFromFile(GBFileItem *fileItem);

/* Returns a new rotation value for the specified degree change. Degrees must be a positive or negative multiple of 90. */
NSInteger GBChangeRotationByDegrees(NSInteger oldRotation, NSInteger degrees);
NSInteger GBDegreesForRotation(NSInteger rotation);
NSInteger GBRotationForDegrees(NSInteger degrees);

#endif /* __GBUTILITYFUNCTIONS__ */

